from xen.xend.server.DevController import DevController
from xen.xend.XendLogging import log

from xen.xend.XendError import VmError
from xen.xend import sxp
import xen.xend
import os

def spawn_detached(path, args, env):
    p = os.fork()
    if p == 0:
        os.spawnve(os.P_NOWAIT, path, args, env)
        os._exit(0)
    else:
        os.waitpid(p, 0)
        
class VfbifController(DevController):
    """Virtual frame buffer controller. Handles all vfb devices for a domain.
    Note that we only support a single vfb per domain at the moment.
    """

    def __init__(self, vm):
        DevController.__init__(self, vm)

    def getDeviceDetails(self, config):
        """@see DevController.getDeviceDetails"""
        type       = sxp.child_value(config, 'type')
        vncunused  = sxp.child_value(config, 'vncunused')
        vncdisplay = sxp.child_value(config, 'vncdisplay')
        vnclisten  = sxp.child_value(config, 'vnclisten')
        display    = sxp.child_value(config, 'display')
        xauthority = sxp.child_value(config, 'xauthority')

        devid = 0

        if type not in ('sdl', 'vnc'):
            raise VmError('Unknown vfb type %s (%s)' % (type, repr(config)))

        front = {}

        back = { 'type' : type }
        if vncunused:
            back['vncunused'] = vncunused
        if vncdisplay:
            back['vncdisplay'] = vncdisplay
        if vnclisten:
            back['vnclisten'] = vnclisten
        if display:
            back['display'] = display
        if xauthority:
            back['xauthority'] = xauthority

        return (devid, back, front)

    def configuration(self, devid):
        r = DevController.configuration(self, devid)
        (type, vncunused, vncdisplay, vnclisten, display, xauthority) \
               = self.readBackend(devid, 'type', 'vncunused', 'vncdisplay',
                                  'vnclisten', 'display', 'xauthority')
        if type:
            r.append(['type', type]),
        if vncunused:
            r.append(['vncunused', vncunused])
        if vncdisplay:
            r.append(['vncdisplay', vncdisplay])
        if vnclisten:
            r.append(['vnclisten', vnclisten])
        if display:
            r.append(['display', display])
        if xauthority:
            r.append(['xauthority', xauthority])
        return r
        
    def createDevice(self, config):
        DevController.createDevice(self, config)
        # old frontend compatibility
        self.vm.writeDom("console/use_graphics", "1")
        # /old
        std_args = [ "--domid", "%d" % self.vm.getDomid(),
                     "--title", self.vm.getName() ]
        t = sxp.child_value(config, "type")
        if t == "vnc":
            passwd = None
            if sxp.child_value(config, "vncpasswd") != None:
                passwd = sxp.child_value(config, "vncpasswd")
            else:
                passwd = xen.xend.XendRoot.instance().get_vncpasswd_default()
            if not(passwd is None or passwd == ""):
                self.vm.storeVm("vncpasswd", passwd)
                log.debug("Stored a VNC password for vfb access")
            else:
                log.debug("No VNC passwd configured for vfb access")

            # Try to start the vnc backend
            args = [xen.util.auxbin.pathTo("xen-vncfb")]
            if sxp.child_value(config, "vncunused") is not None:
                args += ["--unused"]
            elif sxp.child_value(config, "vncdisplay") is not None:
                args += ["--vncport", "%d" % (5900 + sxp.child_value(config, "vncdisplay"))]
            vnclisten = sxp.child_value(config, 'vnclisten',
                                        xen.xend.XendRoot.instance().get_vnclisten_address())
            args += [ "--listen", vnclisten ]
            spawn_detached(args[0], args + std_args, os.environ)
        elif t == "sdl":
            args = [xen.util.auxbin.pathTo("xen-sdlfb")]
            env = dict(os.environ)
            if sxp.child_value(config, "display") is not None:
                env['DISPLAY'] = sxp.child_value(config, "display")
            if sxp.child_value(config, "xauthority") is not None:
                env['XAUTHORITY'] = sxp.child_value(config, "xauthority")
            spawn_detached(args[0], args + std_args, env)
        else:
            raise VmError('Unknown vfb type %s (%s)' % (t, repr(config)))

class VkbdifController(DevController):
    """Virtual keyboard controller. Handles all vkbd devices for a domain.
    """

    def getDeviceDetails(self, config):
        """@see DevController.getDeviceDetails"""
        devid = 0
        back = {}
        front = {}
        return (devid, back, front)
